/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 */

var oSlashdotObserver = null;
var xmlLoader = document.implementation.createDocument("", "xmlload", null);

xmlLoader.addEventListener("load", parseXML, false);

function SlashdotObserver(oPanel)
{
	this.m_oPanel = oPanel;
	this.m_oArticles = new Array();
	this.m_sLinkColor = "#0000FF";
	this.refreshNeeded = true;
	this.m_oTempIFrame = oPanel.getElementById("tempiframe");
}

SlashdotObserver.prototype.observe = function (oSubject, sTopic, sData)
{
	if (sTopic == "handleSelection")
		handleSelection(sData);

	if (oSubject != window._content) // We are only interested in the currently active tab
		return;
	if (sTopic != "EndDocumentLoad")
		return;
	lDomain = oSubject.document.domain.split(".");
	if (lDomain[lDomain.length-2] != "slashdot")
		return;
	
	this.gatherInformation();
	this.refreshContent();
}

SlashdotObserver.prototype.gatherInformation =
function ()
{
	setStatus("Gathering information...");

	this.m_sLinkColor = window._content.document.linkColor;

	oAllTables = window._content.document.getElementsByTagName("Table");
	oArticleTables = new Array();
	for (i = 0; i < oAllTables.length; i++) { // Filter out tables we don't want
		if (matchArticleStructure(oAllTables[i]))
			oArticleTables[oArticleTables.length] = oAllTables[i];
	}

	if (oArticleTables.length > 0) {
		this.refreshNeeded = true;
		this.m_oArticles = new Array();
	}
	for (i = 0; i < oArticleTables.length; i++) {
		oTable = oArticleTables[i];
		oFont = oTable.nextSibling;
		if (oTable.parentNode.nodeName == "P") // A current bug(?) in the Slashdot code that makes the first article embedded in a <p> tag
			oP = oTable.parentNode.nextSibling.nextSibling;
		else
			oP = oTable.nextSibling.nextSibling.nextSibling;

		oBold = oTable.firstChild.firstChild.firstChild.lastChild.firstChild; // Who said anything about clean code?

		sHeadline = oBold.lastChild.nodeValue;
		if (sHeadline[0] == ':') sHeadline = sHeadline.substring(2,sHeadline.length);

		sLink = oP.firstChild.childNodes.item(2).href;

		sThreshCommentsCands = oP.getElementsByTagName("A");
		for (j = 0; j < sThreshCommentsCands.length; j++) {
			if (sThreshCommentsCands.item(j).nextSibling != null && sThreshCommentsCands.item(j).nextSibling.nodeValue == " of ")
				break;
		}
		if (j == sThreshCommentsCands.length) {
			sThreshComments = "0";
			sTotalComments = "0";
		} else {
			sThreshComments = sThreshCommentsCands.item(j).firstChild.nodeValue;
			sTotalComments = sThreshCommentsCands.item(j+1).firstChild.nodeValue;
		}

		if (oFont.firstChild.nodeName == "A" && oFont.firstChild.firstChild.nodeName == "IMG") { // If icons are being shown
			sImg = oFont.firstChild.firstChild.src;
			sImgW = oFont.firstChild.firstChild.width;
			sImgH = oFont.firstChild.firstChild.height;
		} else {
			sImg = "";
			sImgW = "0";
			sImgH = "0";
		}

		if (sImg != "" && oFont.childNodes.item(1).nodeName == "B")
			sPoster = oFont.childNodes.item(1).childNodes.item(1).firstChild.nodeValue;
		else if (oFont.firstChild.nodeName == "B")
			sPoster = oFont.firstChild.childNodes.item(1).firstChild.nodeValue;

		this.m_oArticles[this.m_oArticles.length] = new SlashdotArticle(oTable, sHeadline, sLink, sThreshComments, sTotalComments, sImg, sImgW, sImgH, sPoster, "", "");
	}

	if (this.m_oArticles.length == 0) {
		setStatus("Loading XML file...");
		xmlLoader.load("http://slashdot.org/slashdot.xml");
	}
	else
		setStatus("Done");
}

function parseXML() { if (oSlashdotObserver) oSlashdotObserver.parseXML(); }

SlashdotObserver.prototype.parseXML = 
function ()
{
	this.m_oArticles = new Array();
	lStories = xmlLoader.getElementsByTagName("story");
	for (i = 0; i < lStories.length; i++) {
		sHeadline = lStories.item(i).getElementsByTagName("title").item(0).firstChild.nodeValue;
		sLink = lStories.item(i).getElementsByTagName("url").item(0).firstChild.nodeValue;
		sTime = lStories.item(i).getElementsByTagName("time").item(0).firstChild.nodeValue;
		sPoster = lStories.item(i).getElementsByTagName("author").item(0).firstChild.nodeValue;
		sDepartment = lStories.item(i).getElementsByTagName("department").item(0).firstChild.nodeValue;
		sTotalComments = lStories.item(i).getElementsByTagName("comments").item(0).firstChild.nodeValue;
		sImg = "http://images.slashdot.org/topics/" + lStories.item(i).getElementsByTagName("image").item(0).firstChild.nodeValue;
		this.m_oArticles[this.m_oArticles.length] = new SlashdotArticle(null, sHeadline, sLink, null, sTotalComments, sImg, "50", "40", sPoster, sTime, sDepartment);
	}
	this.refreshContent();
	setStatus("Done");
}

SlashdotObserver.prototype.refreshContent =
function ()
{
	if (this.m_oArticles.length > 0) {
		var oIFrame = this.m_oPanel.getElementById("contentiframe");
		var oTempIFrame = this.m_oTempIFrame;
		var varnames = new Array("linkcolor", "articles", "zoomToHeadline", "prefs");
		var varvalues = new Array(this.m_sLinkColor, this.m_oArticles, zoomToHeadline, prefs);
		if (prefs.getPref("articlemode") == 0)
			parseSZCode(NORMALARTICLECODE, oIFrame.contentDocument, oIFrame.contentWindow, varnames, varvalues);
		else
			parseSZCode(prefs.getPref("articlecode"), oIFrame.contentDocument, oIFrame.contentWindow, varnames, varvalues);
		/*parseSZCode("<html><body>\n%for (var i=0; i < 10; i++)\n<div>TESTICLES!</div>\n</body></html>", oTempIFrame.contentDocument, oTempIFrame.contentWindow, varnames, varvalues);
		var temp = oTempIFrame.contentDocument.firstChild.cloneNode(true);
		this.clearHeader();
		this.m_oPanel.getElementById("headerbox").appendChild(temp);*/
	}
}

SlashdotObserver.prototype.gotoArticle =
function (iArticle)
{
	if (this.m_oArticles[iArticle].m_oZoomTo)
		this.m_oArticles[iArticle].m_oZoomTo.scrollIntoView(true);
}

SlashdotObserver.prototype.clearHeader =
function ()
{
	var oHeader = this.m_oPanel.getElementById("headerbox");
	while (oHeader.hasChildNodes())
		oHeader.removeChild(oHeader.firstChild);
}

function zoomToHeadline(artNum)
{
	oSlashdotObserver.gotoArticle(artNum);
}

function refresh()
{
	if (oSlashdotObserver) {
		oSlashdotObserver.gatherInformation();
		oSlashdotObserver.refreshContent();
	}
}

function prefsChanged()
{
	prefs.loadPrefs();
	if (oSlashdotObserver)
		oSlashdotObserver.refreshContent();
}

function timedRefresh()
{
	refresh();
	setTimeout("timedRefresh()", 1000 * 60 * 30); // Run again in 30 minutes, will add preference for this later (but 30mins will be the minimum delay)
}

function initSlashdot()
{
	var oObsService = Components.classes["@mozilla.org/observer-service;1"].getService();
	oObsInterface = oObsService.QueryInterface(Components.interfaces.nsIObserverService);

	oSlashdotObserver = new SlashdotObserver(document);

	if (oObsInterface && oSlashdotObserver) {
		oObsInterface.addObserver(oSlashdotObserver, "EndDocumentLoad", false);
	}

	timedRefresh();
}

function killSlashdot()
{
	var oObsService = Components.classes["@mozilla.org/observer-service;1"].getService();
	oObsInterface = oObsService.QueryInterface(Components.interfaces.nsIObserverService);
	if (oObsInterface && oSlashdotObserver) {
		oObsInterface.removeObserver(oSlashdotObserver, "EndDocumentLoad");
		oSlashdotObserver = null;
	}
}

function setStatus(status)
{
	if (oSlashdotObserver) {
		var statBar = oSlashdotObserver.m_oPanel.getElementById("status");
		statBar.value = status;
	}
}

addEventListener("load", initSlashdot, false);
addEventListener("unload", killSlashdot, false);
